library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity tirage is

    port (
        I_clk         : in  std_logic;
        I_rst         : in  std_logic;
        I_clk_display : in  std_logic;
        I_bouton      : in  std_logic;
        O_reg0        : out std_logic_vector(5 downto 0);
        O_reg1        : out std_logic_vector(5 downto 0);
        O_reg2        : out std_logic_vector(5 downto 0);
        O_reg3        : out std_logic_vector(5 downto 0);
        O_reg4        : out std_logic_vector(5 downto 0);
        O_reg5        : out std_logic_vector(5 downto 0);
        O_l_rouge     : out std_logic
        O_l_rouge     : out std_logic
        );

end tirage;

architecture a_tirage of tirage is

    component automate is
        port (
            rst, clk                           : in  std_logic;
            clk_display, bouton, invalide, fin : in  std_logic;
            comptage                           : out std_logic;
            enregistrement                     : out std_logic;
            l_rouge                            : out std_logic;
            l_verte                            : out std_logic
            );
    end component;


    component registres is
        port (
            clk, rst                           : in  std_logic;
            enregistre                         : in  std_logic;
            adr                                : in  std_logic_vector(2 downto 0);
            numero_courant                     : in  std_logic_vector(5 downto 0);
            reg0, reg1, reg2, reg3, reg4, reg5 : out std_logic_vector(5 downto 0)
            );
    end component;

    component compteur_valid is
        port (
            clk, rst, comptage : in  std_logic;
            adr                : out std_logic_vector(2 downto 0);
            fin                : out std_logic
            );
    end component;


    component comparateur is
        port (
            reg0, reg1, reg2, reg3, reg4 : in  std_logic_vector(5 downto 0);
            nombre_courant               : in  std_logic_vector(5 downto 0);
            invalide                     : out std_logic
            );
    end component;


    component compteur1_49 is
        port (
            clk, rst, comptage : in  std_logic;
            sortie             : out std_logic_vector(5 downto 0)
            );
    end component;

    component led_pwm is
        port(
            clk, rst           : in  std_logic;
            ledR, ledV         : in  std_logic;
            ledR_PWM, ledV_PWM : out std_logic
            );
    end component led_pwm;

    signal recom, comptage                        : std_logic;
    signal enregistrement, invalide, fin          : std_logic;
    signal adr                                    : std_logic_vector(2 downto 0);
    signal numero_courant, r0, r1, r2, r3, r4, r5 : std_logic_vector(5 downto 0);

    signal l_V, l_R               : std_logic;
    signal cpt_leds, cpt_leds_reg : unsigned(4 downto 0);

begin

    automate_1 : entity work.automate
        port map (
            rst            => rst,
            clk            => clk,
            clk_display    => clk_display,
            bouton         => bouton,
            invalide       => invalide,
            fin            => fin,
            comptage       => comptage,
            enregistrement => enregistrement,
            l_rouge        => l_R,
            l_verte        => l_V
            );

    registres_1 : entity work.registres
        port map (
            clk            => clk,
            rst            => rst,
            enregistre     => enregistrement,
            adr            => adr,
            numero_courant => numero_courant,
            reg0           => r0,
            reg1           => r1,
            reg2           => r2,
            reg3           => r3,
            reg4           => r4,
            reg5           => r5
            );

    compteur_valid_1 : entity work.compteur_valid
        port map (
            clk      => clk,
            rst      => rst,
            comptage => enregistrement,
            adr      => adr,
            fin      => fin
            );

    comparateur_1 : entity work.comparateur
        port map (
            reg0           => r0,
            reg1           => r1,
            reg2           => r2,
            reg3           => r3,
            reg4           => r4,
            nombre_courant => numero_courant,
            invalide       => invalide
            );

    compteur_1 : entity work.compteur1_49
        port map (
            clk      => clk,
            rst      => rst,
            comptage => comptage,
            sortie   => numero_courant
            );


    led_pwm_1 : entity work.led_pwm
        port map (
            clk      => clk,
            rst      => rst,
            ledR     => l_R,
            ledV     => l_V,
            ledR_PWM => l_rouge,
            ledV_PWM => l_verte
            );

    reg0 <= r0;
    reg1 <= r1;
    reg2 <= r2;
    reg3 <= r3;
    reg4 <= r4;
    reg5 <= r5;
end a_tirage;
